<?php

/**
 * Hub endpoint.
 */
function push_hub_endpoint() {

  // These parameters are required in the pubsubhubbub spec.
  // http://pubsubhubbub.googlecode.com/svn/trunk/pubsubhubbub-core-0.3.html
  // @todo hub_verify is required in the spec but this module only supports immediate verification.
  $required_parameters = array(
    'hub_callback',
    'hub_mode',
    'hub_topic',
    'hub_verify',
  );

  // Instead of passing around the full $_POST directly, build up an array
  // that only contains parameters in the pubsubhubub spec.
  $passable_post = array();

  // Assume that $_POST has required paramters. Set to false when one is found
  // to be missing.
  $has_required_parameters = TRUE;

  // if required parameters are missing, build up an array for the error message.
  $missing_parameters = array();

  // A boolean for whether the subscribe/unsubscribe request was successful.
  $success = FALSE;

  // Build up $passable_post
  foreach ($required_parameters as $value) {
    if (!empty($_POST[$value])) {
      $passable_post[$value] = $_POST[$value];
    }
    else {
      $has_required_parameters = FALSE;
      $missing_parameters[] = $value;
    }
  }

  // Check that no required parameters were found to be missing.
  if ($has_required_parameters === TRUE) {

    // These parameters are in the pubsubhubbub spec but not required.
    $optional_parameters = array (
      'hub_lease_seconds',
      'hub_secret',
      'hub_verify_token',
    );

    // Add optional parameters to $passable_post.
    foreach ($optional_parameters as $value) {
      if (!empty($_POST[$value])) {
        $passable_post[$value] = $_POST[$value];
      }
    }

    // hub_mode should only be 'subscribe' or 'unsubscribe'.
    switch($passable_post['hub_mode']) {
      case 'subscribe':
        $success = push_hub()->subscribe($passable_post);
        break;
      case 'unsubscribe':
        $success = push_hub()->unsubscribe($passable_post);
        break;
      default:
        watchdog('push_hub', 'subscription invalid and not saved. hub_mode was not "subscribe" or "unsubscribe".', NULL, WATCHDOG_DEBUG);
        $success = FALSE;
        break;
    }
  }
  else {
    watchdog('push_hub', 'subscription invalid and not saved. POST did not contain all required parameters. Missing ' . implode(', ', $missing_parameters), NULL, WATCHDOG_DEBUG);
    $success = FALSE;
  }

  // If the post was succesful, set a 204. Otherwise set a 404.
  if ($success === TRUE) {
    header('HTTP/1.1 204 "No Content"', null, 204);
  }
  else {
    header('HTTP/1.1 404 "Not Found"', null, 404);
  }

  // The request is complete. Exit.
  drupal_exit();
}
